==> free-sa-1.2.1.tar.gz: free-sa-1.2.1/src/logformats/l_qmail_syslog.c <==
#include "global.h"

/* UNOPTIMIZED */

#define F_QMSLREADRECORD "QMSLReadRecordI"
struct MailRecord *QMSLReadRecordI(FILE *iLog) {
    char tBuf[9216];
    char *trs;
    struct MailRecord *t = NULL, tMR;
    
    
    /* read next string from log */
    while (fgets(tBuf, sizeof(tBuf), iLog) != NULL) {
    	/* find beginning of string (for qmail it's own datetime) */
	if ((trs = strchr(&tBuf[SYSLOG_DT_LEN], ':')) == NULL) continue; /* incorrect record */
	else trs += 2L;
    
	/* info msg */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"info")) {
	    tMR.name[0] = '-';
	    
	    if (sscanf(trs + 27L, "%u%*8c %lf %*[^<]%*c%127[^>]", &tMR.msgid, &tMR.bytes, tMR.name) >= 2) {
		t = FindNewMR(tMR.msgid);
		if (tMR.name[0] != '-') strcpy(t->name, tMR.name);
		t->bytes = tMR.bytes;
		t->status = 1;
	    }
	} else
	
	/* starting delivery */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"star")) {
	    if (sscanf(trs + 36L, "%hd%*6c%u%*4c%*s %s", &tMR.code, &tMR.msgid, tMR.url) == 3)
		if ((t = FindDupMR(tMR.msgid)) != NULL) {
		    strcpy(t->url, tMR.url);
		    t->code = tMR.code;
		    t->status = 2;
		}
	} else
	
	/* delivery */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"deli")) {
	    if (sscanf(trs, "%ld%*17c%hd%*2c%c", &tMR.time, &tMR.code, &tMR.stat) == 3)
		if ((t = FindMRbyCode(tMR.code)) != NULL) {
		    switch (tMR.stat) {
			case 's': t->code = 250; break;
			case 'd': t->code = 451; break;
			case 'f': t->code = 550; break;
			 default: t->code = 600; SAwarning(F_QMSLREADRECORD, "Unknown stat: %c, please contact developers", tMR.stat); break;
		    }
		    
		    strcpy(t->ip, "127.0.0.1");
		    t->time = tMR.time;
		    t->count = 1;
		    t->stat = 0;
		    t->status = 3;
		    
		    return t;
		}
	} else
	
	/* end msg */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"end "))
	    if (sscanf(trs + 26L, "%u", &tMR.msgid) == 1) DelMRs(tMR.msgid);
    }

    return NULL;
}

==> free-sa-1.4.3.tar.gz: free-sa-1.4.3/src/logformats/l_qmail_syslog.c <==
#include "global.h"


/* UNOPTIMIZED */
struct MailRecord *QMSLReadRecordI(FILE *iLog) {
    char tBuf[RECORDSIZE];
    char *trs;
    struct MailRecord *t = NULL, tMR;
    
    
    /* read next string from log */
    while (fgets(tBuf, sizeof(tBuf), iLog) != NULL) {
    	/* find beginning of string (for qmail it's own datetime) */
	if ((trs = strchr(&tBuf[SYSLOGDTSIZE], ':')) == NULL) continue; /* incorrect record */
	else trs += 2L;
    
	/* info msg */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"info")) {
	    tMR.name[0] = '-';
	    
	    if (sscanf(trs + 27L, "%u%*8c %lf %*[^<]%*c%" USERNAMELEN "[^>]", &tMR.msgid, &tMR.bytes, tMR.name) >= 2) {
		t = FindNewMR(tMR.msgid);
		if (tMR.name[0] != '-') strcpy(t->name, tMR.name);
		t->bytes = tMR.bytes;
		t->status = 1;
	    }
	} else
	
	/* starting delivery */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"star")) {
	    if (sscanf(trs + 36L, "%hd%*6c%u%*4c%*s %" URLLEN "s", &tMR.code, &tMR.msgid, tMR.url) == 3)
		if ((t = FindDupMR(tMR.msgid)) != NULL) {
		    strcpy(t->url, tMR.url);
		    t->code = tMR.code;
		    t->status = 2;
		}
	} else
	
	/* delivery */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"deli")) {
	    if (sscanf(trs, "%ld%*17c%hd%*2c%c", &tMR.time, &tMR.code, &tMR.stat) == 3)
		if ((t = FindMRbyCode(tMR.code)) != NULL) {
		    switch (tMR.stat) {
			case 's': t->code = 250; break;
			case 'd': t->code = 451; break;
			case 'f': t->code = 550; break;
			 default: t->code = 600; SAwarning("Unknown stat: %c, please contact developers", tMR.stat); break;
		    }
		    
		    strcpy(t->ip, "127.0.0.1");
		    t->time = tMR.time;
		    t->count = 1;
		    t->stat = 0;
		    t->status = 3;
		    
		    return t;
		}
	} else
	
	/* end msg */
	if (*(const int *)(trs + 18L) == *(const int *)((const char *)"end "))
	    if (sscanf(trs + 26L, "%u", &tMR.msgid) == 1) DelMRs(tMR.msgid);
    }

    return NULL;
}
